// biblioteke

#include <NewPing.h>

// defines

#define MINREADING 14
#define MAXREADING 1009
#define STARTINGVAL -1500

// globalne varijable

const int button_1_Pin = 2;    // touchbutton 1
const int button_2_Pin = 5;    // touchbutton 2
const int pot_1_Pin = A0;      // potenciometar 1 (sliding)
const int pot_2_Pin = A1;      // potenciometar 2 (rotacijski)
const int pot_3_Pin = A2;      // potenciometar 3 (VOLUME - sliding)
const int ultraEchoPin = 3;    // echo za ultrasonic
const int ultraTrigPin = 4;    // trig za ultrasonic

// varijable touchbutton

int last_Button_1_State = 0;
int last_Button_2_State = 0;

// varijable potenciometar

int last_Pot_1_Value = STARTINGVAL;
int last_Pot_2_Value = STARTINGVAL;
int last_Pot_3_Value = STARTINGVAL;

// varijable ultrasonic

bool object_detected = false;

// timing varijable

unsigned long button_1_last_time = millis();
unsigned long button_2_last_time = millis();

unsigned long ultrasonic_last_time = millis();

// prototipi funkcija

void potenciometar(int Pin, int &lastValue, const int sensitivity, String ID);
void touchbutton(const int Pin, const int &lastState, const String ID);
void ultrasonic(bool &object_detected);

// ultrasonic

NewPing sonar(ultraTrigPin, ultraEchoPin);

// setup funkcija
void setup() {
  pinMode(button_1_Pin, INPUT);
  pinMode(button_2_Pin, INPUT);
  pinMode(pot_1_Pin, INPUT);
  pinMode(pot_2_Pin, INPUT);
  pinMode(pot_3_Pin, INPUT);

  Serial.begin(115200);

  while(!Serial);
}

// -------------------- GLAVNI LOOP --------------------

void loop() {
  // POTENCIOMETRI

    potenciometar(pot_1_Pin, last_Pot_1_Value, 5, "POT_1:");
    potenciometar(pot_2_Pin, last_Pot_2_Value, 5, "POT_2:");
    potenciometar(pot_3_Pin, last_Pot_3_Value, 5, "POT_3:");
  
  // TOUCHBUTTONI

  if(millis() - button_1_last_time >= 30)
  {
    button_1_last_time = millis();
    touchbutton(button_1_Pin, last_Button_1_State, "Button_1");
  }
  
  if(millis() - button_2_last_time >= 31)
  {
    button_2_last_time = millis();
    touchbutton(button_2_Pin, last_Button_2_State, "Button_2");
  }

  // ULTRASONIC
  
  if(millis() - ultrasonic_last_time >= 51)
  {
    ultrasonic_last_time = millis();
    ultrasonic(12, object_detected);
  }
}

// -------------------- FUNKCIJE --------------------

void potenciometar(int Pin, int &lastValue, const int sensitivity, String ID)
{
  int currentValue = analogRead(Pin);
  
  int diff = currentValue - lastValue;
  
  int mappedCurrentValue = map(currentValue, MINREADING, MAXREADING, 0, 100);
  mappedCurrentValue = constrain(mappedCurrentValue, 0, 100);
  
  int mappedLastValue = map(lastValue, MINREADING, MAXREADING, 0, 100);
  mappedLastValue = constrain(mappedLastValue, 0, 100);
  
  if ((mappedCurrentValue != mappedLastValue) && (abs(diff) > sensitivity))
  {
    Serial.print(ID);
    Serial.println(mappedCurrentValue);
    lastValue = currentValue;
  }
  else if (lastValue == STARTINGVAL)
  {
    Serial.print(ID);
    Serial.println(mappedCurrentValue);
    lastValue = currentValue;
  }
}

void touchbutton(int Pin, int &lastState, String ID)
{
  int currentState = digitalRead(Pin);
  
  if (currentState != lastState)
  {
    if (currentState == HIGH)
    {
      Serial.print(ID);
      Serial.println("_on");
    }
    else
    {
      Serial.print(ID);
      Serial.println("_off");
    }
    lastState = currentState;
  }
}

void ultrasonic(const int max_distance, bool &object_detected)
{
  int distance = sonar.ping_cm();
  if ((distance > 0) && (distance < max_distance) && (object_detected == false)){
    object_detected = true;
    Serial.println("Ultrasonic_found");
    //Serial.println(distance);
  }
  else if ((distance > max_distance) && (object_detected == true))
  {
    object_detected = false;
    Serial.println("Ultrasonic_lost");
  }
}
