import serial
import time
import subprocess
import json
import psutil
import pyautogui
from xdo import Xdo


"""
Funkcija dohvaća ime prozora koji je trenutno fokusiran
"""
def get_current_window():
    try:
        active_window = xdo.get_pid_window(xdo.get_active_window())
        active_window = psutil.Process(active_window).name()
        return active_window
    except:
        pass

"""
Funkcija izvodi skripte koje koriste vrijednost potenciometra očitanu pri pokretanju programa
"""
def potentiometer_setup(**kwargs):
    setup_script = kwargs['data']['setup'][kwargs['senzor']]
    if setup_script:
        try:
            subprocess.run(setup_script + ' ' + kwargs['value'], shell=True, check=True, timeout=10)
        except:
            pass
    else:
        pass

"""
Funkcija izvodi odgovarajuću akciju (pokreće program ili simulira pritisak tipke na tipkovnici) ako je korisnik pomaknuo klizač potenciometra.

Ulazni argumenti:
'data' - dictionary sa podacima
'active_window' - ime trenutno fokusiranog prozora
'senzor' - komponenta s kojom korisnik trenutno manipulira (potenciometar 1,2 ili 3)
'direction' - smjer u kojem je klizač potenciometra pomaknut 'up' ili 'down'
"""
def potentiometer_action(**kwargs):
    which_action = kwargs['data']['apps'][kwargs['active_window']][kwargs['senzor']]['which']

    if which_action == 'keypress':
        potentiometer_keypress = kwargs['data']['apps'][kwargs['active_window']][kwargs['senzor']]['keypress'][kwargs['direction']]
        if 'meta' in potentiometer_keypress:
            potentiometer_keypress = potentiometer_keypress.replace('meta','winleft')
        else:
            pass
        potentiometer_keypress = potentiometer_keypress.split('+')

        if potentiometer_keypress:
            try:
                pyautogui.hotkey(*potentiometer_keypress)
            except ValueError:
                pass
        else:
            pass

    elif which_action == 'script':
        potentiometer_script = kwargs['data']['apps'][kwargs['active_window']][kwargs['senzor']]['script'][kwargs['direction']]
        if potentiometer_script:
            try:
                subprocess.run(potentiometer_script, shell=True, check=True, timeout=10)
            except:
                pass
        else:
            pass

"""
Funkcija izvodi odgovarajuću akciju (pokreće program ili simulira pritisak tipke na tipkovnici) ako je korisnik interaktirao s senzorima dodira ili udaljenosti.

Ulazni argumenti:
'data' - dictionary sa podacima
'active_window' - ime trenutno fokusiranog prozora
'senzor' - komponenta s kojom korisnik trenutno manipulira (senzori dodira, senzor udaljenosti)
"""
def single_action(**kwargs):

    which_action = kwargs['data']['apps'][kwargs['active_window']][kwargs['senzor']]['which']

    if which_action == 'keypress':
        button_keypress = kwargs['data']['apps'][kwargs['active_window']][kwargs['senzor']]['keypress']
        if 'meta' in button_keypress:
            button_keypress = button_keypress.replace('meta','winleft')
        else:
            pass
        button_keypress = button_keypress.split('+')

        if button_keypress:
            try:
                pyautogui.hotkey(*button_keypress)
            except ValueError:
                pass
        else:
            pass
    elif which_action == 'script':
        button_script = kwargs['data']['apps'][kwargs['active_window']][kwargs['senzor']]['script']
        if button_script:
            try:
                subprocess.run(button_script, shell=True, check=True, timeout=10)
            except ValueError:
                pass
        else:
            pass

# -----------------------------------------------------------------------------------------------------------------------------------------------------------
# otvara konfiguracijsku datoteku i zapisuje njen sadržaj u dictionary 'data'
with open('config.json') as config_file:
    data = json.load(config_file)

# dictionary koji sadrži zadnje očitane vrijednosti pojedinih potenciometara
last_values = {
    'POT_1' : 0,
    'POT_2' : 0,
    'POT_3' : 0
}

# inicijalizacija objekta klase Serial, ujedno otvara serijski port pri baudrate-u 1200 što resetira razvojnu pločicu 
ser = serial.Serial('/dev/ttyACM0', 1200, timeout=30)

# uvjet koji mijenja baudrate serijskog porta na ispravnu vrijednost nakon 10 sekundi (nešto više od vremena potrebnog za reset pločice)
if ser.is_open == True:
    ser.close()

    time.sleep(10)

    ser.baudrate = 115200
    ser.open()
else:
    print("Greska pri otvaranju porta")

# nakon pokretanja pločica uvijek pošalje 3 linije na serijski port, te linije sadrže inicijalne vrijednosti potenciometara koje se predaju funkciji potentiometer_setup() koja ih zatim koristi u startup skriptama
for i in range(3):
    line = str(ser.readline())
    try:
        event, value = line[2:][:-5].split(':')
        last_values[event] = int(value)
        try:
            potentiometer_setup(data=data, senzor=event, value=value)
        except KeyError:
            pass
    except ValueError:
        exit(1)

# inicijalizacija objekta klase Xdo(), objekt se koristi u klasi get_current_window()
xdo = Xdo()

# glavna petlja programa
while(True):
    # čitanje linije serijske komunikacije
    line = str(ser.readline())

    # određivanje trenutno fokusiranog prozora
    active_window = get_current_window()
    
    # linija koja prikazuje primljenu poruku
    # print(line[2:][:-5])

    # uvjet koji provjerava postoji li u konfiguracijskoj datoteci (čiji je sadržaj u 'data' varijabli) konfiguracija za trenutno fokusirani prozor te postavlja vrijednost varijable 'active_window' na 'global' ukoliko konfiguracija ne postoji
    if active_window in data['apps']:
        pass
    else:
        active_window = 'global'
    
    # linija koja dešifrira i odvaja poruku očitanu sa serijskog porta (očitana linija prije ovog postupka sadrži znakove za kraj linije koji se ovim postupkom uklanjaju)
    try:
        event, value = line[2:][:-5].split(':')
    except ValueError:
        event = line[2:][:-5]
        value = 0

    # uvjet koji izvodi odgovarajuću funkciju zavisno o poruci koja je očitana na serijskom portu (senzor i/ili vrijednost senzora s kojim je korisnik interaktirao)
    if event[0:][:3] == 'POT':
        # uvjet koji detektira u kojem smjeru je pomaknut potenciometar
        if int(value) > last_values[event]:
            direction = 'up'
        elif int(value) < last_values[event]:
            direction = 'down'
        
        # dvije linije koje služe za ispravljanje situacije u kojoj je potenciometar pomaknut no poruka nije poslana/primljena u serijskoj komunikaciji (čest problem pri prebrzom pomicanju klizača potenciometra)
        magic_number = 1 if last_values[event] < int(value) else -1

        l = sum(1 for v in range(last_values[event], int(value) + magic_number, magic_number) if v != last_values[event] and v % data['apps'][active_window][event]['steps'] == 0)

        for i in range(l):
            try:
                # poziv funkcije koja izvodi odgovarajuću akciju
                potentiometer_action(data=data, active_window=active_window, senzor=event, direction=direction)
            except KeyError:
                pass
        
        # zapisuje se zadnja očitana vrijednost potenciometra u odgovarajući dictionary
        last_values[event] = int(value)

    elif event[0:][:3] != '':
        try:
            single_action(data=data, active_window=active_window, senzor=event)
        except KeyError:
            pass